import 'package:flutter/material.dart';
import 'package:flutter_simple/src/simple/components/border_simple.dart';
import 'package:flutter_simple/src/simple/components/gradient_simple.dart';
import 'package:flutter_simple/src/simple/components/radius_simple.dart';
import 'package:flutter_simple/src/simple/components/shadow_simple.dart';

import 'container_simple.dart';

class ButtonSimple extends StatefulWidget {
  final Widget child;

  final bool enabled;
  final bool useSkeleton;
  final bool isCenter;
  final bool isExpanded;
  // final bool elevation;

  final EdgeInsets padding;
  final EdgeInsets margin;
  final double height;
  final double width;

  final DecorationImage image;

  final Color color;
  final Color disabledBackgroundColor;

  final Alignment alignment;
  final Function onPressed;
  final Function onLongPress;

  //effects
  final Color splashColor;
  final Color highlightColor;
  final Color focusColor;
  final Color hoverColor;

  // SimplePackage
  final RadiusSimple radius;
  final BorderSimple border;
  final List<ShadowSimple> shadow;
  final GradientSimple gradient;

  ButtonSimple({
    Key key,
    this.child,
    this.padding,
    this.margin,
    this.height,
    this.width,
    this.color = Colors.transparent,
    this.disabledBackgroundColor,
    this.alignment,
    //bools
    this.isCenter,
    this.isExpanded,
    this.enabled,
    this.useSkeleton,
    // actions
    this.onPressed,
    this.onLongPress,
    this.splashColor,
    this.highlightColor,
    this.focusColor,
    this.hoverColor,
    // Simple
    this.radius,
    this.border,
    this.shadow,
    this.gradient,
    this.image,
  });
  // : this.super(key: key);

  @override
  _ButtomSimpleState createState() => _ButtomSimpleState();
}

class _ButtomSimpleState extends State<ButtonSimple> {
  // bool showElevation;

  ContainerSimple _containerSimple;

  @override
  void initState() {
    // this.showElevation = this.widget.elevation;
    super.initState();
  }

  @override
  void didUpdateWidget(ButtonSimple oldWidget) {
    // this.showElevation = this.widget.elevation;
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return this._buildButtomSimple();
  }

  Widget _buildButtomSimple() {
    _containerSimple = ContainerSimple(
      padding: EdgeInsets.all(0),
      margin: this.widget.margin,
      isCenter: this.widget.isCenter,
      isExpanded: this.widget.isExpanded,
      alignment: this.widget.alignment,
      color: this.isEnabled
          ? this.widget.color
          : this.widget.disabledBackgroundColor ??
              this.widget.color.withOpacity(0.4),
      height: this.widget.height,
      width: this.widget.width,
      // useSkeleton: this.widget.useSkeleton,
      border: this.widget.border,
      gradient: this.isEnabled ? this.widget.gradient : null,
      radius: this.widget.radius,
      shadows: this.widget.shadow,
      // notifyParent: () => setState(() {}),
      child: this._buildChildButton(),
    );
    return _containerSimple;
  }

  _buildChildButton() {
    return FlatButton(
      padding: this.widget.useSkeleton == true
          ? EdgeInsets.all(0)
          : this.widget.padding,
      splashColor: this.widget.splashColor,
      highlightColor: this.widget.highlightColor,
      focusColor: this.widget.focusColor,
      hoverColor: this.widget.hoverColor,
      // sempre transparente para usar a cor do container
      color: Colors.transparent,
      onHighlightChanged: (value) {},
      onPressed: this.isEnabled ? this.widget.onPressed : null,
      onLongPress: this.isEnabled ? this.widget.onLongPress : null,
      child: Center(
        child: this.widget.child,
      ),
    );
  }

  // Widget _buildElevationContainer({@required Widget buttonChild}) {
  //   return Container(
  //     foregroundDecoration: _containerSimple?.containerSize == null
  //         ? null
  //         : BoxDecoration(
  //             boxShadow: showElevation == true
  //                 ? [
  //                     BoxShadow(
  //                       color: Colors.black26,
  //                       spreadRadius: -2,
  //                       blurRadius: 8,
  //                       offset: Offset(
  //                           0, _containerSimple.containerSize.height - 10.0),
  //                     ),
  //                     BoxShadow(
  //                       color: Colors.black26,
  //                       spreadRadius: -2,
  //                       blurRadius: 8,
  //                       offset: Offset(
  //                           _containerSimple.containerSize.width - 10.0, 0),
  //                     ),
  //                   ]
  //                 : null,
  //           ),
  //     child: buttonChild,
  //   );
  // }

  bool get isEnabled {
    if ((this.widget.onPressed == null && this.widget.onLongPress == null) ||
        this.widget.enabled == false)
      return false;
    else
      return true;
  }
}
