import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_simple/src/simple/components/border_simple.dart';
import 'package:flutter_simple/src/simple/components/gradient_simple.dart';
import 'package:flutter_simple/src/simple/components/radius_simple.dart';
import 'package:flutter_simple/src/simple/components/shadow_simple.dart';
import 'package:flutter_simple/src/simple/components/text_decoration_simple.dart';
import 'package:flutter_simple/src/simple/components/text_field_mask.dart';
import 'package:flutter_simple/src/simple/widgets/text_simple.dart';

import 'container_simple.dart';

class TextFieldSimple extends StatelessWidget {
  // only number

  final EdgeInsets padding;
  final EdgeInsets margin;
  final double height;
  final double width;
  final Alignment alignment;

  final Color backgroundColor;

  final bool useSkeleton;
  final bool isCenter;
  final bool isExpanded;

  // SimplePackage
  final RadiusSimple radius;
  final BorderSimple border;
  final List<ShadowSimple> shadow;
  final GradientSimple gradient;

  //TextField
  final TextEditingController controller;
  final FocusNode focusNode;
  final TextInputType textInputType;
  final TextCapitalization textCapitalization;
  final TextAlign textAlign;
  final TextAlignVertical textAlignVertical;

  final Function(String) onChaned;
  final Function onTap;
  final Function onEditingComplete;
  final Function onSubmitted;

  final String mask;

  final TextSimple placeholder;

  final TextInputAction textInputAction;

  final Color cursorColor;

  final int maxLength;
  final int maxLines;
  final double cursorRadius;
  final double cursorWidth;

  final bool enabled;
  final bool autocorrect;
  final bool autofocus;
  final bool obscureText;
  final bool readOnly;
  final bool showCursor;
  final bool onlyNumbers;

  // style
  final Color textColor;
  final double fontSize;
  final FontWeight fontWeight;
  final FontStyle fontStyle;
  final double letterSpacing;
  final double wordSpacing;
  final String fontFamily;
  final TextDecorationSimple textDecorationSimple;

  final Widget prefixWidget;
  final Widget suffixWidget;
  // placeholderStyle: this._buildTextStyle(),

  TextFieldSimple({
    this.enabled,
    this.useSkeleton,
    this.isCenter,
    this.isExpanded,
    this.padding,
    this.margin,
    this.height = 50,
    this.width,
    this.radius,
    this.border,
    this.shadow,
    this.gradient,
    this.alignment,
    this.backgroundColor,
    this.controller,
    this.textInputType,
    this.textCapitalization = TextCapitalization.none,
    this.textAlign = TextAlign.start,
    this.mask,
    this.maxLength,
    this.maxLines,
    this.autocorrect = false,
    this.autofocus = false,
    this.obscureText = false,
    this.onlyNumbers = false,
    this.focusNode,
    this.onChaned,
    this.readOnly = false,
    this.cursorColor,
    this.textAlignVertical = TextAlignVertical.center,
    this.textColor,
    this.fontSize,
    this.fontWeight,
    this.fontStyle,
    this.letterSpacing,
    this.wordSpacing,
    this.fontFamily,
    this.textDecorationSimple,
    this.placeholder,
    this.cursorRadius = 2,
    this.cursorWidth = 2,
    this.showCursor = true,
    this.prefixWidget,
    this.suffixWidget,
    // this.placeholderStyle,
    this.onTap,
    this.onEditingComplete,
    this.onSubmitted,
    this.textInputAction = TextInputAction.done,
  });

  static ContainerSimple _containerSimple;

  @override
  Widget build(BuildContext context) {
    return this._buildContainerSimple();
  }

  ContainerSimple _buildContainerSimple() {
    _containerSimple = ContainerSimple(
      padding: this.useSkeleton == true
          ? EdgeInsets.all(0)
          : this.padding ?? EdgeInsets.symmetric(horizontal: 4),
      margin: this.margin,
      isCenter: this.isCenter,
      isExpanded: this.isExpanded,
      alignment: this.alignment,
      color: this.backgroundColor,
      height: this.height,
      width: this.width,
      // useSkeleton: this.useSkeleton,
      border: this.border,
      gradient: this.gradient,
      radius: this.radius,
      shadows: this.shadow,
      // notifyParent: () => setState(() {}),
      child: this._buildTextField(),
    );
    return _containerSimple;
  }

  Widget _buildTextField() {
    return CupertinoTextField(
      padding: EdgeInsets.all(0),
      autocorrect: this.autocorrect,
      enabled: this.enabled,
      controller: this.controller,
      focusNode: this.focusNode,
      autofocus: this.autofocus,
      textCapitalization: this.textCapitalization,
      keyboardAppearance: Brightness.dark,
      cursorColor: this.cursorColor,
      textAlign: TextAlign.start,
      obscureText: this.obscureText,
      readOnly: this.readOnly,
      textAlignVertical: this.textAlignVertical,
      keyboardType: this.onlyNumbers == true && this.textInputType == null
          ? TextInputType.number
          : this.textInputType,
      maxLines: this.maxLines,
      maxLength: this.maxLength,
      maxLengthEnforced: true,
      showCursor: this.showCursor,
      cursorRadius: Radius.circular(this.cursorRadius),
      cursorWidth: this.cursorWidth,
      placeholder: this.placeholder?.text,
      placeholderStyle: this._buildPlaceholderTextStyle(),
      style: this._buildTextStyle(),
      inputFormatters: this._getFormatters(),
      prefix: this.prefixWidget,
      suffix: this.suffixWidget,
      onChanged: this.onChaned,
      onTap: this.onTap,
      onEditingComplete: this.onEditingComplete,
      onSubmitted: this.onSubmitted,
      textInputAction: this.textInputAction,

      // remove native decoration
      decoration: null,
    );
  }

  TextStyle _buildTextStyle() {
    return TextStyle(
      backgroundColor: Colors.transparent,
      color: this.textColor,
      fontSize: this.fontSize,
      fontWeight: this.fontWeight,
      fontStyle: this.fontStyle,
      decoration: this.textDecorationSimple?.textDecoration,
      decorationColor: this.textDecorationSimple?.color,
      decorationStyle: this.textDecorationSimple?.decorationStyle,
      decorationThickness: this.textDecorationSimple?.decorationThickness,
      letterSpacing: this.letterSpacing,
      fontFamily: this.fontFamily,
      wordSpacing: this.wordSpacing,
    );
  }

  TextStyle _buildPlaceholderTextStyle() {
    if (this.placeholder == null) return null;
    return TextStyle(
      backgroundColor: this.placeholder.backgroundColor,
      color: this.placeholder.color,
      fontSize: this.placeholder.fontSize,
      fontWeight: this.placeholder.fontWeight,
      fontStyle: this.placeholder.fontStyle,
      decoration: this.placeholder.textDecorationSimple?.textDecoration,
      decorationColor: this.placeholder.textDecorationSimple?.color,
      decorationStyle: this.placeholder.textDecorationSimple?.decorationStyle,
      decorationThickness:
          this.placeholder.textDecorationSimple?.decorationThickness,
      letterSpacing: this.placeholder.letterSpacing,
      fontFamily: this.placeholder.fontFamily,
      wordSpacing: this.placeholder.wordSpacing,
      height: this.placeholder.height,
    );
  }

  List<TextInputFormatter> _getFormatters() {
    List<TextInputFormatter> textInputFormatters = [];
    if (this.maxLength != null) {
      var maxLengthFormatter = LengthLimitingTextInputFormatter(this.maxLength);
      textInputFormatters.add(maxLengthFormatter);
    }
    if (this.mask != null) {
      var maskFormatter = new TextFieldMaskSimple(
          mask: this.mask, filter: {"#": RegExp(r'[0-9]')});
      textInputFormatters.add(maskFormatter);
    }

    if (this.onlyNumbers == true)
      textInputFormatters.add(WhitelistingTextInputFormatter.digitsOnly);

    return textInputFormatters;
  }
}
