import 'package:flutter/material.dart';
import 'package:flutter_simple/flutter_simple.dart';
import 'package:flutter_simple/src/simple/components/shadow_simple.dart';

class TextSimple extends StatelessWidget {
  // TODO
  // urls
  // phones
  // dates

  // atributos
  final String text;
  final int maxLines;
  final String mask;
  final bool isCenter;
  final bool isExpandend;
  final bool isCleanText; // fazer

  // style
  final TextOverflow textOverflow;
  final TextAlign textAlign;
  final TextDirection textDirection;
  final Color color;
  final double fontSize;
  final FontWeight fontWeight;
  final FontStyle fontStyle;
  final Color backgroundColor;
  final double letterSpacing;
  final double wordSpacing;
  final double height;
  final String fontFamily;

  final TextDecorationSimple textDecorationSimple;
  final List<ShadowSimple> shadows;

  TextSimple(
    this.text, {
    Key key,
    this.maxLines,
    this.textOverflow,
    this.textAlign = TextAlign.left,
    this.textDirection = TextDirection.ltr,
    // style
    this.color = Colors.black,
    this.fontSize,
    this.fontWeight = FontWeight.normal,
    this.fontStyle = FontStyle.normal,
    this.backgroundColor = Colors.transparent,
    this.letterSpacing,
    this.wordSpacing,
    this.height,
    this.fontFamily,
    // custom
    this.mask,
    this.isCenter = false,
    this.isExpandend = false,
    this.isCleanText,
    this.textDecorationSimple,
    this.shadows,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    Widget textSimple = this._buildTextSimple();

    if (this.isCenter == true)
      textSimple = Center(
        child: textSimple,
      );

    if (this.isExpandend == true)
      textSimple = Expanded(
        child: textSimple,
      );

    return textSimple;
  }

  Widget _buildTextSimple() {
    return Text(
      this._maskText,
      key: this.key,
      maxLines: this.maxLines,
      overflow: this.textOverflow,
      textAlign: this.textAlign,
      textDirection: this.textDirection,
      style: this.textStyle,
    );
  }

  TextStyle get textStyle {
    return TextStyle(
      color: this.color,
      fontSize: this.fontSize,
      fontWeight: this.fontWeight,
      fontStyle: this.fontStyle,
      backgroundColor: this.backgroundColor,
      decoration: this.textDecorationSimple?.textDecoration,
      decorationColor: this.textDecorationSimple?.color,
      decorationStyle: this.textDecorationSimple?.decorationStyle,
      decorationThickness: this.textDecorationSimple?.decorationThickness,
      letterSpacing: this.letterSpacing,
      fontFamily: this.fontFamily,
      wordSpacing: this.wordSpacing,
      height: this.height,
      shadows: this._buildShadows(),
    );
  }

  String get _maskText {
    try {
      if (this.text == null || this.text.isEmpty)
        return '';
      else if (this.mask == null || this.mask.isEmpty)
        return this.text;
      else {
        int maskItemCount = 0;
        String maskedString = '';
        for (var i = 0; i < mask.length; i++) {
          if (mask[i] == '#')
            maskedString += this._cleanText[i - maskItemCount];
          else {
            maskedString += mask[i];
            maskItemCount++;
          }
        }
        return maskedString;
      }
    } catch (e) {
      print(e);
      return '* invalid mask *';
    }
  }

  String get _cleanText {
    return this
        .text
        .replaceAll(new RegExp(r'[^\w\s]+'), '')
        .replaceAll(' ', '');
  }

  _buildShadows() {
    if (this.shadows == null || this.shadows.isEmpty)
      return null;
    else
      return this.shadows.map((SimpleShadow) {
        return Shadow(
          color: SimpleShadow.color.withOpacity(SimpleShadow.opacity),
          blurRadius: SimpleShadow.blur,
          offset: Offset(
            SimpleShadow.moveRight,
            SimpleShadow.moveDown,
          ),
        );
      }).toList();
  }
}
